/*
 * access.c - implementation of access methods
 * by Jonathan Kay, Computer Systems Lab, UCSD         8/3/93
 */
/*
 * Copyright (c) 1993,1994 Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice and the following
 * paragraph appears in all copies of this software.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO
 * EVENT SHALL THE REGENTS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* Note: this implementation assumes only one access function, kerndig */

#include <stdio.h>
#include "vars.h"
#include "configurable.h"

extern int doprint;

/* Kernel access interface */
extern struct var *kreadq, *kwriteq;
extern int nkread, nkwrite;
extern int existwarn;

extern int ninames();

access_init()
{
    kernel_init();
}

access_exec()
{
    if (kwriteq)
	kernel_set(kwriteq, nkwrite);
    if (kreadq)
	kernel_get(kreadq, nkread);
}

access_readq(v)
Var *v;
{
    /* Implement existwarn policy.  Turn off for multiple writes. */
    if (kreadq)
	existwarn = 0;

    /* Add variable to queue */
    v->next = kreadq;
    kreadq = v;
    nkread += ninames(v);
    v->value = (char *) malloc(v->len);
    v->accflags |= VAF_READ;
    doprint = 1;
}

access_writeq(v, vstr)
Var *v;
char *vstr;
{
    int value;
    char *ep;

    /* Implement existwarn policy.  Turn off for multiple writes. */
    if (kwriteq)
	existwarn = 0;

    /* Add variable to queue */
    v->next = kwriteq;
    kwriteq = v;
    nkwrite += ninames(v);
    v->accflags |= VAF_WRITE;
    v->value = (char *) malloc(v->len);

    switch(v->type) {
      case VT_INT:
	value = strtol(vstr, &ep, 10);
	if (ep == vstr) {
	    fprintf(stderr, "Argument to <%s> must be an integer\n", v->xname);
	    exit(1);
	}
	break;
	
      case VT_BOOL:
	if (vstr[0] == 'o') {
	    if (!strcmp(vstr, "on"))
		value = 1;
	    else if (!strcmp(vstr, "off"))
		value = 0;
	    else {
		fprintf(stderr, "Argument to <%s> must be 'on,' 'off,' or an integer\n",
			v->xname);
		exit(1);
	    }
	}
	else {
	    value = strtol(vstr, &ep, 10);
	    if (ep == vstr) {
		fprintf(stderr, "Argument to <%s> must be 'on,' 'off,' or an integer\n",
			v->xname);
		exit(1);
	    }
	}
	break;
	
      default:
	fprintf(stderr, "%s has unknown type %d (netconfig is misconfigured)!\n",
		v->xname, v->type);
	exit(1);
    }
    
    *(int *) v->value = value;
}

static ninames(v)
Var *v;
{
    char **inamep;
    int n = 0;

    for (inamep = v->inames; *inamep; ++inamep)
	++n;
    return(n);
}
